### CREDITS FOR TUTORIAL
### FROM http://planatscher.net/svmtut/svmtut.html
## CLEAN SWEEP
## rm(list = ls())
### LOAD THE e1071 PACKAGE 
require(e1071)


###-------------------------------------
### SVM EXAMPLE 1
###-------------------------------------

### CREATE TRAINING DATA
data1 <- seq(1,10,by=2)
classes1 <- c('a','a','a','b','b')
classes1 <- as.factor(classes1)

### CREATE TEST DATA
test1 <- seq(1,10,by=2) + 1

### TRAIN THE MODEL
model1 <- svm(data1,classes1,type='C',kernel='linear')

### PRINT THE RESULTS OF THE MODEL
print(model1)

## SUMMARY
summary(model1)

## STRUCTURE
str(model1)

## SPECIFIC ATTR OF THE MODEL
model1$decision.values


### USE THE MODEL TO PREDICT CLASSES USING THE TRAINING DATA
predict(model1,data1)
pred <- fitted(model1)

## CONFUSION MATRIX
table(pred, classes1)	

####---------------------------------------
### SVM EXAMPLE 2: NOT LINEAR SEP.
####---------------------------------------
data2 <- seq(21,30)

classes2 <- c('b','b','b','a','a','a','a','b','b','b')
classes2 <- as.factor(classes2)

## LINEAR MODEL
model2 <- svm(data2,classes2,type='C',kernel='linear')


predict(model2,data2)

## CONFUSION MATRIX
table(predict(model2,data2), classes2)

### USE A RADIAL KERNEL ON SECOND PASS TO SEE HOW THE KERNEL IMPACTS THE MODEL
model2 <- svm(data2,classes2,type='C',kernel='radial')

predict(model2,data2)

## CONFUSION MATRIX
table(predict(model2,data2), classes2)


####-----------------------
#### BREAST CANCER EXAMPLE
####-----------------------
## Download the data  (http://www.potschi.de/svmtut/breast-cancer-wisconsin.data), and save it in the directory from where you started the R-term.

## Read the data from the CSV-file.

setwd("E:/TEMP/201001/")
bcdata <- read.csv('breast-cancer-wisconsin.data',head=TRUE)

### Take a look at the column titles. 
### Which feature does *not* contain information useful for 
### classification?

names(bcdata)


#####
###  SOME R STUFF: ADDRESSING ITEMS IN A DATA FRAME EXAMPLES
#####
str(bcdata)
head(bcdata)
tail(bcdata)
bcdata[1,]
bcdata[1,2]
bcdata$ClumpThickness
bcdata$ClumpThickness[2]

### Separate the data from the classification. 
### The subset()-funtion allows you to (un)select the right columns..

databcall <- subset(bcdata,select=c(-Samplecodenumber,-Class))
classesbcall <- subset(bcdata,select=Class)

### Take a subset of your data, that you will use as training data.

databctrain <- databcall[1:400,]
classesbctrain <- as.factor(classesbcall[1:400,])

## Take a subset of your data, that you will use as test data.

databctest <- databcall[401:699,]
classesbctest <- as.factor(classesbcall[401:699,])

## Build the model.
model <- svm(databctrain, classesbctrain, type='C')

## PREDICTION
pred <- predict(model, databctrain)
table(pred,t(classesbctrain))

## RUN WITH THE TEST DATA
pred <- predict(model, databctest)
table(pred,t(classesbctest))



### TUNING

## *** TUNING REQUIRES ALL THE DATA IN ONE DATAFRAME
tune_bc_DF <- cbind(databctrain, classesbctrain)

tune_bc_model <-  tune.svm(classesbctrain~., data=tune_bc_DF, gamma = 2^(-1:1), cost = 2^(2:4))

### SUMMARY
summary(tune_bc_model)

### PLOT GRID
plot(tune_bc_model)

###  SHOW THE PROPERTIES OF THE TUNE OBJECT
### str(tune_bc_model)
### BEST COST = tune_bc_model$best.model$cost
### BEST GAMMA = tune_bc_model$best.model$gamma

### WITH THE TUNING PARMS
model <- svm(databctrain, classesbctrain, gamma=tune_bc_model$best.model$gamma, cost=tune_bc_model$best.model$cost)
pred <- predict(model, databctrain)

## CONFUSION MATRIX
table(pred,t(classesbctrain))

## Validate the model. Does it work?

pred <- predict(model, databctest)
table(pred,t(classesbctest))


